@@ -13,7 +13,7 @@
 #
 
 """Listener action implementation"""
-
+import argparse
 
 from cliff import lister
 from osc_lib.command import command
@@ -26,7 +26,8 @@ from octaviaclient.osc.v2 import constants as const
 from octaviaclient.osc.v2 import utils as v2_utils
 from octaviaclient.osc.v2 import validate
 
-PROTOCOL_CHOICES = ['TCP', 'HTTP', 'HTTPS', 'TERMINATED_HTTPS', 'UDP', 'SCTP']
+PROTOCOL_CHOICES = ['TCP', 'HTTP', 'HTTPS', 'TERMINATED_HTTPS', 'UDP', 'SCTP',
+                    'PROMETHEUS']
 CLIENT_AUTH_CHOICES = ['NONE', 'OPTIONAL', 'MANDATORY']
 
 
@@ -200,6 +201,34 @@ class CreateListener(command.ShowOne):
             help="Set the ALPN protocol to be used "
                  "by the listener (can be set multiple times)."
         )
+        parser.add_argument(
+            '--hsts-max-age',
+            dest='hsts_max_age',
+            metavar='<hsts_max_age>',
+            type=int,
+            help="The value of the max_age directive for the "
+                 "Strict-Transport-Security HTTP response header. "
+                 "Setting this enables HTTP Strict Transport "
+                 "Security (HSTS) for the TLS-terminated listener."
+        )
+        parser.add_argument(
+            '--hsts-include-subdomains',
+            action='store_true',
+            dest='hsts_include_subdomains',
+            default=None,
+            help="Define whether the includeSubDomains directive should be "
+                 "added to the Strict-Transport-Security HTTP response "
+                 "header."
+        )
+        parser.add_argument(
+            '--hsts-preload',
+            action='store_true',
+            dest='hsts_preload',
+            default=None,
+            help="Define whether the preload directive should be "
+                 "added to the Strict-Transport-Security HTTP response "
+                 "header."
+        )
 
         _tag.add_tag_option_to_parser_for_create(
             parser, 'listener')
@@ -229,12 +258,12 @@ class CreateListener(command.ShowOne):
                         data['listener']['id']))
             }
 
-        formatters = {'loadbalancers': v2_utils.format_list,
-                      'pools': v2_utils.format_list,
-                      'l7policies': v2_utils.format_list,
-                      'insert_headers': v2_utils.format_hash,
-                      'allowed_cidrs': v2_utils.format_list_flat,
-                      'tags': v2_utils.format_list_flat}
+        formatters = {'loadbalancers': v2_utils.ListColumn,
+                      'pools': v2_utils.ListColumn,
+                      'l7policies': v2_utils.ListColumn,
+                      'insert_headers': v2_utils.HashColumn,
+                      'allowed_cidrs': v2_utils.FlatListColumn,
+                      'tags': v2_utils.FlatListColumn}
 
         return (rows,
                 (utils.get_dict_properties(data['listener'],
@@ -323,7 +352,7 @@ class ListListener(lister.Lister):
         attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                             parsed_args)
         data = self.app.client_manager.load_balancer.listener_list(**attrs)
-        formatters = {'loadbalancers': v2_utils.format_list}
+        formatters = {'loadbalancers': v2_utils.ListColumn}
         return (columns,
                 (utils.get_dict_properties(s, columns, formatters=formatters)
                  for s in data['listeners']))
@@ -361,12 +390,12 @@ class ShowListener(command.ShowOne):
             data = self.app.client_manager.load_balancer.listener_show(
                 listener_id=listener_id,
             )
-        formatters = {'loadbalancers': v2_utils.format_list,
-                      'pools': v2_utils.format_list,
-                      'l7policies': v2_utils.format_list,
-                      'insert_headers': v2_utils.format_hash,
-                      'allowed_cidrs': v2_utils.format_list_flat,
-                      'tags': v2_utils.format_list_flat}
+        formatters = {'loadbalancers': v2_utils.ListColumn,
+                      'pools': v2_utils.ListColumn,
+                      'l7policies': v2_utils.ListColumn,
+                      'insert_headers': v2_utils.HashColumn,
+                      'allowed_cidrs': v2_utils.FlatListColumn,
+                      'tags': v2_utils.FlatListColumn}
 
         return rows, utils.get_dict_properties(data, rows,
                                                formatters=formatters)
@@ -527,6 +556,35 @@ class SetListener(command.Command):
             help="Set the ALPN protocol to be used "
                  "by the listener (can be set multiple times)."
         )
+        parser.add_argument(
+            '--hsts-max-age',
+            dest='hsts_max_age',
+            metavar='<hsts_max_age>',
+            type=int,
+            default=argparse.SUPPRESS,
+            help="The value of the max_age directive for the "
+                 "Strict-Transport-Security HTTP response header. "
+                 "Setting this enables HTTP Strict Transport "
+                 "Security (HSTS) for the TLS-terminated listener."
+        )
+        parser.add_argument(
+            '--hsts-include-subdomains',
+            action='store_true',
+            default=argparse.SUPPRESS,
+            dest='hsts_include_subdomains',
+            help="Defines whether the includeSubDomains directive should be "
+                 "added to the Strict-Transport-Security HTTP response "
+                 "header."
+        )
+        parser.add_argument(
+            '--hsts-preload',
+            action='store_true',
+            default=argparse.SUPPRESS,
+            dest='hsts_preload',
+            help="Defines whether the preload directive should be "
+                 "added to the Strict-Transport-Security HTTP response "
+                 "header."
+        )
 
         _tag.add_tag_option_to_parser_for_set(parser, 'listener')
 
@@ -663,7 +721,27 @@ class UnsetListener(command.Command):
             action='store_true',
             help="Clear all ALPN protocols from the listener."
         )
-
+        parser.add_argument(
+            '--hsts-max-age',
+            dest='hsts_max_age',
+            action='store_true',
+            help="Disables HTTP Strict Transport "
+                 "Security (HSTS) for the TLS-terminated listener."
+        )
+        parser.add_argument(
+            '--hsts-include-subdomains',
+            action='store_true',
+            dest='hsts_include_subdomains',
+            help="Removes the includeSubDomains directive from the "
+                 "Strict-Transport-Security HTTP response header."
+        )
+        parser.add_argument(
+            '--hsts-preload',
+            action='store_true',
+            dest='hsts_preload',
+            help="Removes the preload directive from the "
+                 "Strict-Transport-Security HTTP response header."
+        )
         _tag.add_tag_option_to_parser_for_unset(parser, 'listener')
 
         return parser
