@@ -9,12 +9,12 @@
 #   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #   License for the specific language governing permissions and limitations
 #   under the License.
-#
+
+import argparse
 import copy
 import itertools
 from unittest import mock
 
-import munch
 from osc_lib import exceptions
 from oslo_utils import uuidutils
 
@@ -456,6 +456,35 @@ class TestLoadBalancerCreate(TestLoadBalancer):
         self.api_mock.load_balancer_create.assert_called_with(
             json={'loadbalancer': lb_info})
 
+    @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
+    def test_load_balancer_create_with_vip_sg_ids(self, mock_client):
+        vip_sg_ids = [
+            uuidutils.generate_uuid(),
+            uuidutils.generate_uuid()
+        ]
+        lb_info = copy.deepcopy(self.lb_info)
+        lb_info.update({'vip_sg_ids': vip_sg_ids})
+        mock_client.return_value = lb_info
+
+        arglist = [
+            '--name', self._lb.name,
+            '--vip-network-id', self._lb.vip_network_id,
+            '--project', self._lb.project_id,
+            '--vip-sg-id', vip_sg_ids[0],
+            '--vip-sg-id', vip_sg_ids[1],
+        ]
+        verifylist = [
+            ('name', self._lb.name),
+            ('vip_network_id', self._lb.vip_network_id),
+            ('project', self._lb.project_id),
+            ('vip_sg_id', vip_sg_ids),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.api_mock.load_balancer_create.assert_called_with(
+            json={'loadbalancer': lb_info})
+
     @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
     def test_load_balancer_create_with_tags(self, mock_client):
         lb_info = copy.deepcopy(self.lb_info)
@@ -496,9 +525,12 @@ class TestLoadBalancerCreate(TestLoadBalancer):
                 # subtract comb's keys from attrs_list
                 filtered_attrs = {k: v for k, v in attrs_list.items() if (
                     k not in comb)}
+
                 # Add the 'wait' attribute, which isn't part of an LB directly
                 filtered_attrs['wait'] = False
                 mock_client.return_value = filtered_attrs
+                parsed_args = argparse.Namespace(**filtered_attrs)
+
                 if not any(k in filtered_attrs for k in args) or all(
                     k in filtered_attrs for k in ("vip_network_id",
                                                   "vip_port_id")
@@ -506,10 +538,11 @@ class TestLoadBalancerCreate(TestLoadBalancer):
                     self.assertRaises(
                         exceptions.CommandError,
                         self.cmd.take_action,
-                        filtered_attrs)
+                        parsed_args
+                    )
                 else:
                     try:
-                        self.cmd.take_action(munch.Munch(filtered_attrs))
+                        self.cmd.take_action(parsed_args)
                     except exceptions.CommandError as e:
                         self.fail("%s raised unexpectedly" % e)
 
@@ -675,6 +708,24 @@ class TestLoadBalancerSet(TestLoadBalancer):
         except Exception as e:
             self.fail("%s raised unexpectedly" % e)
 
+    @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
+    def test_load_balancer_set_vip_sg_id(self, mock_attrs):
+        mock_attrs.return_value = {
+            'loadbalancer_id': self._lb.id,
+        }
+        sg_id = uuidutils.generate_uuid()
+        arglist = [self._lb.id, '--vip-sg-id', sg_id]
+        verifylist = [
+            ('loadbalancer', self._lb.id),
+            ('vip_sg_id', [sg_id]),
+        ]
+
+        try:
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+            self.cmd.take_action(parsed_args)
+        except Exception as e:
+            self.fail("%s raised unexpectedly" % e)
+
 
 class TestLoadBalancerStats(TestLoadBalancer):
 
@@ -883,3 +934,26 @@ class TestLoadBalancerUnset(TestLoadBalancer):
         self.api_mock.load_balancer_set.assert_called_once_with(
             self._lb.id,
             json={'loadbalancer': {'tags': []}})
+
+    @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
+    def test_load_balancer_unset_vip_sg_id(self, mock_attrs):
+        sg_id = uuidutils.generate_uuid()
+        mock_attrs.return_value = {
+            'loadbalancer_id': self._lb.id,
+            'vip_sg_ids': [sg_id]
+        }
+        arglist = [self._lb.id, '--vip-sg-id']
+        verifylist = [
+            ('loadbalancer', self._lb.id),
+            ('vip_sg_ids', True),
+        ]
+
+        try:
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+            self.cmd.take_action(parsed_args)
+        except Exception as e:
+            self.fail("%s raised unexpectedly" % e)
+
+        self.api_mock.load_balancer_set.assert_called_once_with(
+            self._lb.id,
+            json={'loadbalancer': {'vip_sg_ids': None}})
